<?php
/**
 * This class is designed to show a teacher which assignments contain work
 * that requires marking. It can only be added to the frontpage and if the
 * user is not a teacher or does not have any assignments, the block does not
 * appear. In addition if there is no work still to be marked the block does
 * not appear. Works in Moodle 1.8+
 *
 * the block looks for the grade assignment capability.
 *
 * original author Mark J Tyers [ZANNET]   * @version 31/07/07
 * modified by Bruno Vernier bruno.vernier@gmail.com to include journals and other gradeable activities
 * with thanks to  Open Technology LTD http://www.opentechnology.ru  Authors: Alex Djachenko, Evgenij Tsygantsov
 * for the quiz essay portion of the code
 * modified by Mark J Tyers to only display the correct courses if logged in as an admin user
 */
class block_marking extends block_list {

    function init() {
        $this->title = get_string('blockname', 'block_marking');
        $this->version = 2007062900;
    }
    
    function get_content() {
        // Candidates for Configurable Parameters:
        $coursename = 'shortname' ;  //Mark prefers 'fullname' here; Bruno prefers the more space-frugal 'shortname'
        $maxlength = 20; //for limiting size of titles of activities

        global $CFG, $USER;
        $this->content = new stdClass;
        $this->content->items = array();
        $this->content->icons = array();
        
        $marking_total = 0; // keeps track of total unmarked assignments for footer
        
        $teacher_role=get_field('role','id','shortname','editingteacher'); // retrieve the teacher role id (3)
        $ne_teacher_role=get_field('role','id','shortname','teacher'); // retrieve the non-editing teacher role id (4)
        
        $courses = get_records_select('course', '', 'sortorder', "id, $coursename as fullname");
        foreach($courses as $key=>$val) {
            if ($val->id != 1) {
                $context = get_context_instance(CONTEXT_COURSE, $val->id);
                $i = 0;  //unmarked items counter
                
                $teachers = get_role_users($teacher_role, $context, true); // check for editing teachers
                $j = false;
                if ($teachers) {
                    foreach($teachers as $key2=>$val2) {
                        if ($val2->id == $USER->id) {
                            $j = true;
                        }
                    }
                }
                $teachers_ne = get_role_users($ne_teacher_role, $context, true); // check for non-editing teachers
                if ($teachers_ne) {
                    foreach($teachers_ne as $key2=>$val2) {
                        if ($val2->id == $USER->id) {
                            $j = true;
                        }
                    }
                }
                if (!$j) {continue;} // only display if in a teacher or teacher_non_editing role
                    
                // ASSIGNMENT ============================================================================
                if (has_capability('mod/assignment:grade', $context)) {
                    $select = "course='$val->id'";
                    $assignments = get_records_select('assignment', $select);
                    if ($assignments) {
                      foreach($assignments as $key2=>$val2) {
                        // counts how many students still need their assignment marking
                        $select = 'timemodified > timemarked and assignment = ' . $val2->id;
                        $students = get_record_select('assignment_submissions', $select, 'count(id) as id');
                        // if there is some work needing marking we display the assignment name together with the unmarked number in parenthesis.
                        if ($students->id) {
                            $marking_total += $students->id;
                            $i++;
                            if ($i == 1) { // as soon as we find an assignment we add the course name
                                $this->content->items[] = '<strong>' . shorten_text($val->fullname,$maxlength) . '</strong>';
                                $this->content->icons[]='';
                            }
                            $num = ' (' . $students->id . ')';
                            // finally we retrieve the module instance id so we can create a hyperlink to the marking page.
                            $select2 = "course=$val->id and module=1 and instance=$val2->id";
                            $module = get_record_select('course_modules', $select2, 'id');
                            $url = $CFG->wwwroot.'/mod/assignment/submissions.php?id=' . $module->id . '&tsort=timemodified';
                            $this->content->items[] = '<a href="' . $url . '"><small>' . shorten_text($val2->name,$maxlength) . $num . '</small></a>';
                            $this->content->icons[]='<img src="'.$CFG->wwwroot.'/mod/assignment/icon.gif" class="icon" alt="" />';
                          }
                        }
                    }
                }
                // JOURNAL ====================================================================================
                if (has_capability('mod/assignment:grade', $context)) { // could not find journal capabilities
                    $select = "course='$val->id'";
                    $assignments = get_records_select('journal', $select);
                    if ($assignments) {
                      foreach($assignments as $key2=>$val2) {
                        // counts how many students still need their assignment marking
                        $select = 'modified > timemarked and journal = ' . $val2->id;
                        $students = get_record_select('journal_entries', $select, 'count(id) as id');
                        // if there is some work needing marking we display the assignment name together with the unmarked number in parenthesis.
                        if ($students->id) {
                            $marking_total += $students->id;
                            $i++;
                            if ($i == 1) { // as soon as we find an assignment we add the course name
                                $this->content->items[] = '<strong>' . shorten_text($val->fullname,$maxlength) . '</strong>';
                                $this->content->icons[]='';
                            }
                            $num = ' (' . $students->id . ')';
                            // finally we retrieve the module instance id so we can create a hyperlink to the marking page.
                            $select2 = "course=$val->id and module=9 and instance=$val2->id";
                            $module = get_record_select('course_modules', $select2, 'id');
                            $url = $CFG->wwwroot.'/mod/journal/report.php?id=' . $module->id;
                            $this->content->items[] = '<a href="' . $url . '"><small>' . shorten_text($val2->name,$maxlength) . $num . '</small></a>';
                            $this->content->icons[]='<img src="'.$CFG->wwwroot.'/mod/journal/icon.gif" class="icon" alt="" />';
                          }
                        }
                    }
                }
                // FAQ =======================================================================
                if (has_capability('mod/glossary:grade', $context)) {
                    $select = "course='$val->id'";
                    $assignments = get_records_select('glossary', $select);
                    if ($assignments) {
                      foreach($assignments as $key2=>$val2) {
                        // counts how many students still need their assignment marking
                        $select = 'approved=0 and glossaryid = ' . $val2->id;
                        $students = get_record_select('glossary_entries', $select, 'count(id) as id');
                        // if there is some work needing marking we display the assignment name together with the unmarked number in parenthesis.
                        if ($students->id) {
                            $marking_total += $students->id;
                            $i++;
                            if ($i == 1) { // as soon as we find an assignment we add the course name
                                $this->content->items[] = '<strong>' . shorten_text($val->fullname,$maxlength) . '</strong>';
                                $this->content->icons[]='';
                            }
                            $num = ' (' . $students->id . ')';
                            // finally we retrieve the module instance id so we can create a hyperlink to the marking page.
                            $select2 = "course=$val->id and module=8 and instance=$val2->id";
                            $module = get_record_select('course_modules', $select2, 'id');
                            $url = $CFG->wwwdata.'/mod/glossary/view.php?mode=approval&id=' . $module->id;
                            $this->content->items[] = '<a href="' . $url . '"><small>' . shorten_text($val2->name,$maxlength) . $num . '</small></a>';
                            $this->content->icons[]='<img src="'.$CFG->wwwroot.'/mod/glossary/icon.gif" class="icon" alt="" />';
                          }
                        }
                    }
                }
                // FORUM ======================================================================
                if (has_capability('mod/forum:rate', $context)) { // forum:grade does not exist!
                    $select = " assessed !=0 and course='$val->id'";
                    $forums = get_records_select('forum', $select);
                    if ($forums) {
                        foreach($forums as $forum) {
                            $select = "assessed !=0  and forum = $forum->id and course = $val->id" ;
                            $discussions = get_records_select('forum_discussions', $select );
                            if ($discussions) {
                                 foreach ($discussions as $discussion) {
                                     $select = " discussion = $discussion->id ";
                                     $posts = get_records_select('forum_posts',$select, 'id');
                                     $num = 0;
                                     foreach ($posts as $post) {
                                         $select = " post = $post->id ";
                                         $student = get_record_select('forum_ratings', $select, 'count(id) as marked' );
                                         if ($student->marked==0) {
                                             $marking_total += 1;
                                             $i++;
                                             $num++;
                                             if ($i == 1) {
                                                 $this->content->items[] = '<strong>' . shorten_text($val->fullname,$maxlength) . '</strong>';
                                                 $this->content->icons[]='';
                                             }
                                             $select2 = "course=$val->id and module=4 and instance=$forum->id";
                                             $module = get_record_select('course_modules', $select2, 'id');
                                             $url = $CFG->wwwroot.'/mod/forum/discuss.php?id=' . $discussion->id; // fixed
                                         }
                                     }
                                     if ($num) {
                                         $num = " ($num) ";
                                         $this->content->items[] = '<a href="' . $url . '"><small>' . shorten_text($forum->name,$maxlength) . $num . '</small></a>';
                                         $this->content->icons[]='<img src="'.$CFG->wwwroot.'/mod/forum/icon.gif" class="icon" alt="" />';
                                     }
                                 }
                            }
                        }
                    }
                }
                // QUIZ ESSAYS  ======================================================================
                if (has_capability('mod/quiz:grade', $context)) {
		    require_once ("{$CFG->dirroot}/mod/quiz/locallib.php");
                    $rez = '';
                    $select = " course='$val->id'";
                    $quizzes = get_records_select('quiz', $select);
                    if ($quizzes) {
                        foreach($quizzes as $quiz) {
                            $all_question = array();
 			    $quiz_questions = quiz_questions_in_quiz($quiz->questions);
                            if ($quiz_questions) {
                                $questions = get_records_select('question', "id IN ($quiz_questions)", 'id', 'id, qtype');
                                foreach ($questions as $key=>$q) {
 				    if ($q->qtype != 'essay') { unset($questions[$key]); }
				}
                            }
                            $all_question[$quiz->id] = $questions;
                            foreach($all_question as $quiz_id=>$questions) {
                              if ($questions) {
 			        foreach ($questions as $q) {
				    $attempts = get_records_sql("SELECT qa.* FROM {$CFG->prefix}quiz_attempts qa, {$CFG->prefix}question_sessions qs ".
								"WHERE	quiz = $quiz_id AND qa.timefinish > 0 ".
  							        " AND qa.preview = 0 AND qs.questionid = '$q->id'");
				    if ($attempts) {
				        $data=array(); 
                                        $str = '';
                                        $num = 0;
				        foreach ($attempts as $attempt) {
                                            $graded = '';
				            //$graded = $this->is_graded($attempt, $q->id);
  		                            $sql = "SELECT state.id, state.event, sess.manualcomment
                 				FROM {$CFG->prefix}question_states state, {$CFG->prefix}question_sessions sess WHERE sess.newest = state.id 
									AND sess.attemptid = $attempt->uniqueid	AND sess.questionid = '$q->id'";
                                            $state = get_records_sql($sql);
                         		    if ($state) {
                                                $graded = question_state_is_graded($state) OR $state->manualcomment;
                                                $marking_total += 1;
                                                $i++;
                                                $num++;
                                                if ($i == 1) {
                                                    $this->content->items[] = '<strong>' . shorten_text($val->fullname,$maxlength) . '</strong>';
                                                    $this->content->icons[]='';
                                                }
						//$data[] = array($quiz_id, $q->id, fullname($users[$attempt->userid]), $attempt);
 		                                $this->content->items[] ="<a href=\"{$CFG->wwwroot}/mod/quiz/report.php?mode=grading&amp;q=$quiz_id".
                                                     "&amp;action=viewquestion&amp;questionid=$q->id\">". shorten_text($quiz->name,$maxlength)." ($num) </a>";
                                                $this->content->icons[]='<img src="'.$CFG->wwwroot.'/mod/quiz/icon.gif" class="icon" alt="" />';

					    }
				        }
				    }
                                  }
 				}
                            }
                        }
                    }
                }
            }
        }
        if ($marking_total) { // if we have found any unmarked work we display a total in the footer.
            $this->content->footer = 'total unmarked: ' . $marking_total;
        }
    }

    function applicable_formats() {
        return array('all' => true);
//        return array('site' => true);
    }
    
}
?>